var flags = {
    idle: 0,
    left: 1,
    right: 2,
    jump: 4,
    fall: 8,
    attack: 16,
    die: 32
};
var character = new Actor(160, 404);
function buildCharacter(){
    character.height = 64;
    character.blockRight = character.blockLeft = false;
    character.dropTo = null;
    let a = new Anim("images/idle10x37x64.png", 10, 37, true);
    a.pingPong = true;
    character.anims.push(a);
    a = new Anim("images/RunRight10x46x64.png", 10, 46, true);
    character.anims.push(a);
    a = new Anim("images/RunLeft10x46x64.png", 10, 46, true);
    character.anims.push(a);
    a = new Anim("images/JumpRight6x40x64.png", 6, 40, false);
    character.anims.push(a);
    a = new Anim("images/JumpLeft.png", 6, 40, false);
    character.anims.push(a);
    a = new Anim("images/LandRight.png", 4, 40, false);
    character.anims.push(a);
    a = new Anim("images/LandLeft.png", 4, 40, false);
    character.anims.push(a);
    a = new Anim("images/AttackRight.png", 10, 48, false);
    character.anims.push(a);
    a = new Anim("images/AttackLeft.png", 10, 48, false);
    character.anims.push(a);
    a = new Anim("images/die10x62x64.png", 10, 62, false);
    character.anims.push(a);
    character.state = flags.idle;
    character.blockRight = character.blockLeft = false;
}
character.jump = function() {
    if(this.state === flags.idle || (this.state & flags.jump) || (this.state & flags.fall)) {return;}
    this.state |= flags.jump; // add jump to status
    if(this.state & flags.left){
        this.resetCurrent(4);
    } else {
        this.resetCurrent(3);
    }
    this.moveY = game.shiftDown ? game.jump * 1.3 : game.jump;
};
character.manageActions = function() {
    this.dropTo = null;
    this.collisionDetect();
    if((this.state & flags.jump) || (this.state & flags.fall)){
        this.manageFall();
        return;
    }
    if(this.state === flags.die){
        if(this.moveY === -10){
            if(this.y <= -(this.height)){
                this.moveY = 0;// game over
            }
        } else if(this.anims[9].frame === this.anims[9].default){
            this.moveY = -10;// dispose of the body
        }
        return;  
    }
    if(!this.dropTo){
        this.state |= flags.fall;
        this.moveY = game.gravity;
        this.moveX = 0;
        this.resetCurrent((this.state & flags.right) ? 5 : 6);
        return;
    }
    if(game.ctrlDown){
        if(!(this.state & flags.attack)){
            this.resetCurrent((this.state & flags.right) ? 7 : 8);
            this.state |= flags.attack;
            this.moveX = 0;
        }
        return;
    } else if(this.state & flags.attack){
        this.state = flags.idle; //reset below
    }  
    if(game.rightDown && ! this.blockRight){
        if(!(this.state & flags.right)){
            this.resetCurrent(1);
            this.state = flags.right;
            this.blockLeft = false;
        }
        this.moveX = 5;
    } else if(game.leftDown && !this.blockLeft){
        if(!(this.state & flags.left)){
            this.resetCurrent(2);
            this.state = flags.left;
            this.blockRight = false;
            }
            this.moveX = -5;
    } else {
        if((this.state !== flags.idle)){
            this.resetCurrent(0);
            this.state = flags.idle;
            this.moveX = 0;
        }
    }  
};
character.manageFall = function(){
    this.moveY += (this.moveY >= 14) ? 0 : game.gravity; // TV
    if(this.state & flags.jump){
        if(this.moveY >= 0){
            this.state ^= flags.jump;
            this.state |= flags.fall;
            this.moveX = (this.moveX < 0) ? -2 : 2; // slow f'wd motion
            this.resetCurrent((this.state & flags.right) ? 5 : 6);
        }
    } else {
        if(this.dropTo){
            this.moveY = 0;
            this.y = objects.platforms[this.dropTo.index].y - this.height;
            this.state ^= flags.fall;
            this.resetCurrent((this.state & flags.right)? 1 : 2);
        }
    }
};
character.collision = function(collisions){
    for (let c = 0; c < collisions.length; c++){
        switch(collisions[c].type){
            case "things":
                if(this.state === flags.left || this.state === flags.right){
                    this.block();
                } else if(this.state & flags.fall){
                    this.state ^= flags.fall;
                    this.state |= flags.jump;
                    this.moveY = game.jump / 3;
                }
                break;
            case "coins":
                let n = objects.coins[collisions[c].index];
                if(n.visible){
                    n.collision();
                }
                break;
            case "platform":
                let p = objects.platforms[collisions[c].index];
                if(p.y >= this.y + this.height){
                    this.dropTo = collisions[c];
                } else {
                    // things to tackle next
                }
                break;
            case "water":
                sounds.splash.play();
                break;
            case "spiders":
                let s = objects.spiders[collisions[c].index];
                if(s.state !== flags.die && !(this.state & flags.die)){
                    if(this.state & flags.attack){
                        if((this.state & flags.right) && s.x > this.x){
                            s.die();
                        } else if((this.state & flags.left) && s.x < this.x){
                            s.die();
                        } else {this.state = flags.die;}
                    } else {
                        this.state = flags.die;
                    }
                    if(this.state === flags.die){this.resetCurrent(9);}
                }
                break;
        }
    }
};
character.block = function(){
    this.moveX = 0;
    this.resetCurrent(0);
    if(this.state & flags.right){
        game.rightDown = false;
        this.blockRight = true;
    } else if(this.state & flags.left){
        game.leftDown = false;
        this.blockLeft = true;
    }
    this.state = flags.idle;
};